
/**  
 * @file cpsw_mgmt.c
 *
 * @brief 
 *  This file holds all the Ethernet subsystem (CPSW + MDIO + SGMII) components
 *  initialization and setup code.
 *
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2009, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* C Standard library Include */
#include <string.h>
#include <stdio.h>
#include <c6x.h>
/* Chip Level definitions include */
#include <ti/csl/csl_chip.h>
#include <ti/csl/csl_bootcfgAux.h>

/* CSL EMAC include */
#include <ti/csl/csl_cpsw.h>
#include <ti/csl/csl_cpsgmii.h>
#include <ti/csl/csl_cpsgmiiAux.h>
#include <ti/csl/cslr_cpsgmii.h>
#include <ti/csl/csl_mdio.h>
#include <ti/csl/csl_mdioAux.h>
#include <ti/csl/csl_cptsAux.h>
/* BootCfg module include */
#include <ti/csl/csl_bootcfg.h>
#include <ti/csl/csl_bootcfgAux.h>
#include <ti/csl/csl_chipAux.h>

/** Number of ports in the ethernet subsystem */
#define         NUM_PORTS                   3u

/** Number of MAC/GMII ports in the ethernet switch */
#define         NUM_MAC_PORTS               2u


#define CPSW_LOOPBACK_NONE           0
#define CPSW_LOOPBACK_INTERNAL       1
#define CPSW_LOOPBACK_EXTERNAL       2

Uint32 cpswLpbkMode = CPSW_LOOPBACK_NONE;
#define TSC_getDelay(preTSC) 	((unsigned int)((0xFFFFFFFFl+TSCL)- (unsigned long long)preTSC)+ 1)
#define TSC_count_cycle_from(preTSC) 	(TSC_getDelay(preTSC)- cycle_measure_overhead)
/** ============================================================================
 *   @n@b Init_SGMII
 *
 *   @b Description
 *   @n SGMII peripheral initialization code.
 *
 *   @param[in]  
 *   @n macPortNum      MAC port number for which the SGMII port setup must
 *                      be performed.
 * 
 *   @return
 *   @n None
 * =============================================================================
 */
 /*===============================TSC===================================*/
unsigned int cycle_measure_overhead=50;
unsigned int DSP_Core_Clock_KHz = 1000000;
void calc_cycle_measure_overhead()
{
	unsigned int cycle_cold, cycle_warm;
	cycle_cold= TSCL;
	cycle_cold = TSC_getDelay(cycle_cold);
	cycle_warm= TSCL;
	cycle_warm = TSC_getDelay(cycle_warm);
	cycle_measure_overhead = (cycle_cold + cycle_warm)/2;
}

/* Initialize Time stame counter to measure cycles*/
void TSC_init()
{
	TSCL = 0; 	/* Enable the TSC */
	calc_cycle_measure_overhead();
}

/*delay in millisecond*/
void TSC_delay_ms(Uint32 ms)
{
	volatile unsigned long long preTSC, currentTSC;
	unsigned long long delay_cycles;
	Uint32 tscl, tsch;

	tscl= TSCL;
	tsch= TSCH;
	preTSC= _itoll(tsch,tscl); 
	
	delay_cycles= ((unsigned long long)ms*1000000000000l/DSP_Core_Clock_KHz);

	do
	{
		tscl= TSCL;
		tsch= TSCH;
		currentTSC= _itoll(tsch,tscl); 
	}
	while((currentTSC-preTSC)<delay_cycles);
}

/*delay in microsecond*/
void TSC_delay_us(Uint32 us)
{
	volatile unsigned long long preTSC, currentTSC;
	unsigned long long delay_cycles;
	Uint32 tscl, tsch;

	tscl= TSCL;
	tsch= TSCH;
	preTSC= _itoll(tsch,tscl); 
	
	delay_cycles= ((unsigned long long)us*1000000000/DSP_Core_Clock_KHz);

	do
	{
		tscl= TSCL;
		tsch= TSCH;
		currentTSC= _itoll(tsch,tscl); 
	}
	while((currentTSC-preTSC)<delay_cycles);
}

void CycleDelay (Int32 count)
{
	Uint32					TSCLin;

	if (count <= 0)
		return;

	/* Get the current TSC	*/
	TSCLin = TSCL;

	while ((TSCL  - TSCLin) < (Uint32)count);
}
void Cpsw_ALE_config(void)
{
	//Cpsw_ALE_add_vlan_config(4,0x3,0x7,0x7,0);
	//Cpsw_ALE_add_vlan_config(5,0x5,0x7,0x7,0);
}

void Init_SGMII (Uint32 macPortNum)
{
    CSL_SGMII_ADVABILITY    sgmiiCfg;
    CSL_SGMII_STATUS        sgmiiStatus;

    /* Reset the port before configuring it */
   // CSL_SGMII_doSoftReset (macPortNum);
   // while (CSL_SGMII_getSoftResetStatus (macPortNum) != 0);

     if (macPortNum == 0) {
        /* Hold the port in soft reset and set up
         * the SGMII control register:
         *      (1) Enable Master Mode
         *      (2) Enable Auto-negotiation
         */
        CSL_SGMII_startRxTxSoftReset (macPortNum);
        CSL_SGMII_disableMasterMode (macPortNum);
        CSL_SGMII_enableAutoNegotiation (macPortNum);
        CSL_SGMII_endRxTxSoftReset (macPortNum);

        /* Setup the Advertised Ability register for this port:
         *      (1) Enable Full duplex mode
         *      (2) Enable Auto Negotiation
         *      (3) Enable the Link
         */
       sgmiiCfg.linkSpeed      =   CSL_SGMII_1000_MBPS;
       sgmiiCfg.duplexMode     =   CSL_SGMII_FULL_DUPLEX;
       CSL_SGMII_setAdvAbility (macPortNum, &sgmiiCfg);
        do
        {
            CSL_SGMII_getStatus(macPortNum, &sgmiiStatus);
        } while (sgmiiStatus.bIsLinkUp != 1);

        /* Wait for SGMII Autonegotiation to complete without error */
        do
        {
            CSL_SGMII_getStatus(macPortNum, &sgmiiStatus);
            if (sgmiiStatus.bIsAutoNegError != 0)
                return; /* This is an error condition */
        } while (sgmiiStatus.bIsAutoNegComplete != 1);
    }

    if (macPortNum == 0) {
        /* Hold the port in soft reset and set up
         * the SGMII control register:
         *      (1) Enable Master Mode
         *      (2) Enable Auto-negotiation
         */
        CSL_SGMII_startRxTxSoftReset (macPortNum);
        CSL_SGMII_enableMasterMode (macPortNum);
        CSL_SGMII_disableAutoNegotiation (macPortNum);
        CSL_SGMII_endRxTxSoftReset (macPortNum);

        /* Setup the Advertised Ability register for this port:
         *      (1) Enable Full duplex mode
         *      (2) Enable Auto Negotiation
         *      (3) Enable the Link
         */
        sgmiiCfg.linkSpeed      =   CSL_SGMII_1000_MBPS;
        sgmiiCfg.duplexMode     =   CSL_SGMII_FULL_DUPLEX;
        sgmiiCfg.bLinkUp        =   1;

        //CSL_SGMII_setAdvAbility (macPortNum, &sgmiiCfg);
		#if 0
		CSL_SGMII_enableLoopback (macPortNum);
        do
        {
            CSL_SGMII_getStatus(macPortNum, &sgmiiStatus);
        } while (sgmiiStatus.bIsLinkUp != 1);
		#endif
     }

	if((cpswLpbkMode == CPSW_LOOPBACK_EXTERNAL) || (cpswLpbkMode == CPSW_LOOPBACK_NONE))
				   TSC_delay_ms(1);
    /* All done with configuration. Return Now. */
    return;
}



void Init_SGMII_u_boot (Uint32 macPortNum)
{
    CSL_SGMII_ADVABILITY    sgmiiCfg;
    CSL_SGMII_STATUS        sgmiiStatus;

	if(macPortNum==0)
	{
		*((unsigned int *)0x2090118)=1;//AdvAbility
		*((unsigned int *)0x2090110)=1;//control
	}
	else
	{
		*((unsigned int *)0x2090218)=1;//AdvAbility
		*((unsigned int *)0x2090210)=1;//control
	}	

	do
        {
            CSL_SGMII_getStatus(macPortNum, &sgmiiStatus);
        } while (sgmiiStatus.bIsLinkUp != 1);

        /* Wait for SGMII Autonegotiation to complete without error */
        do
        {
            CSL_SGMII_getStatus(macPortNum, &sgmiiStatus);
            if (sgmiiStatus.bIsAutoNegError != 0)
                return; /* This is an error condition */
        } while (sgmiiStatus.bIsAutoNegComplete != 1);
  
        /* All done with configuration. Return Now. */
    return;
}


/** ============================================================================
 *   @n@b Init_MAC
 *
 *   @b Description
 *   @n This API initializes the CPGMAC Sliver (MAC Port) port.
 *
 *   @param[in]  
 *   @n macPortNum      MAC port number for which the initialization must be done.
 *
 *   @param[in]  
 *   @n macAddress      MAC address to configure on this port.
 * 
 *   @param[in]  
 *   @n mtu             Maximum Frame length to configure on this port.
 *
 *   @return
 *   @n None
 * =============================================================================
 */
int Init_MAC (Uint32 macPortNum, Uint8 macAddress[6], Uint32 mtu)
{
    /* Reset MAC Sliver 0 */            
    CSL_CPGMAC_SL_resetMac (macPortNum);
	//NO delay should cause issue
    while (CSL_CPGMAC_SL_isMACResetDone (macPortNum) != TRUE);

    /* Setup the MAC Control Register for this port:
     *      (1) Enable Full duplex
     *      (2) Enable GMII
     *      (3) Enable Gigabit 
     *      (4) Enable External Configuration. This enables 
     *          the "Full duplex" and "Gigabit" settings to be
     *          controlled externally from SGMII
     *      (5) Don't enable any control/error/short frames
     */
    CSL_CPGMAC_SL_enableFullDuplex (macPortNum);
    CSL_CPGMAC_SL_enableGMII (macPortNum);
    CSL_CPGMAC_SL_enableGigabit (macPortNum);
    CSL_CPGMAC_SL_enableExtControl (macPortNum);
	CSL_CPGMAC_SL_enableRxCSF(macPortNum);
    /* Configure the MAC address for this port */
    //CSL_CPSW_3GF_setPortMACAddress (macPortNum, macAddress);

    /* Configure VLAN ID/CFI/Priority.
     *
     * For now, we are not using VLANs so just configure them
     * to all zeros.
     */
    //CSL_CPSW_3GF_setPortVlanReg (macPortNum, 0, 0, 0);

    /* Configure the Receive Maximum length on this port,
     * i.e., the maximum size the port can receive without
     * any errors.
     *
     * Set the Rx Max length to the MTU configured for the
     * interface.
     */
    CSL_CPGMAC_SL_setRxMaxLen (macPortNum, mtu);  

	//CSL_CPGMAC_SL_enableRxFlowControl(macPortNum);
	//CSL_CPGMAC_SL_enableTxFlowControl(macPortNum);

    /* Done setting up the MAC port */
    return 0;
}
//EMAC control 0x2090904  0x2090944
#define EMAC_MACCONTROL_GIGABIT_ENABLE		(1 << 7)
#define EMAC_MACCONTROL_GIGFORCE		(1 << 17)
void  tci6614_eth_gigabit_enable(void)
{
	unsigned short data;
	unsigned int   CPSWReg;
	if (!tci6614_phy_read(0, 0, &data)) {
		if (data & (1 << 6)) { /* speed selection MSB */
			/*
			 * Check if link detected is giga-bit
			 * If Gigabit mode detected, enable gigbit in MAC
			 */
			 printf("GIG enable!\n");
			CPSWReg = *((unsigned int *)0x2090904);
			*((unsigned int *)0x2090904) = CPSWReg|
				EMAC_MACCONTROL_GIGFORCE |
				EMAC_MACCONTROL_GIGABIT_ENABLE;
		}
	}
}

int Init_MAC_u_boot (Uint32 macPortNum, Uint8 macAddress[6], Uint32 mtu)
{
    /* Reset MAC Sliver 0 */            
   // CSL_CPGMAC_SL_resetMac (macPortNum);
	//NO delay should cause issue
  // while (CSL_CPGMAC_SL_isMACResetDone (macPortNum) != TRUE);

    /* Setup the MAC Control Register for this port:
     *      (1) Enable Full duplex
     *      (2) Enable GMII
     *      (3) Enable Gigabit 
     *      (4) Enable External Configuration. This enables 
     *          the "Full duplex" and "Gigabit" settings to be
     *          controlled externally from SGMII
     *      (5) Don't enable any control/error/short frames
     */
    //CSL_CPGMAC_SL_enableFullDuplex (macPortNum);
    CSL_CPGMAC_SL_enableGMII (macPortNum);
    //CSL_CPGMAC_SL_enableGigabit (macPortNum);
    CSL_CPGMAC_SL_enableExtControl (macPortNum);
	//CSL_CPGMAC_SL_enableRxCSF(macPortNum);
    /* Configure the MAC address for this port */
    //CSL_CPSW_3GF_setPortMACAddress (macPortNum, macAddress);

    /* Configure VLAN ID/CFI/Priority.
     *
     * For now, we are not using VLANs so just configure them
     * to all zeros.
     */
    //CSL_CPSW_3GF_setPortVlanReg (macPortNum, 0, 0, 0);

    /* Configure the Receive Maximum length on this port,
     * i.e., the maximum size the port can receive without
     * any errors.
     *
     * Set the Rx Max length to the MTU configured for the
     * interface.
     */
    CSL_CPGMAC_SL_setRxMaxLen (macPortNum, mtu);  

	//CSL_CPGMAC_SL_enableRxFlowControl(macPortNum);
	//CSL_CPGMAC_SL_enableTxFlowControl(macPortNum);

    /* Done setting up the MAC port */
    return 0;
}

/** ============================================================================
 *   @n@b Init_Switch
 *
 *   @b Description
 *   @n This API sets up the ethernet switch subsystem and its Address Lookup 
 *      Engine (ALE) in "Switch" mode.
 *
 *   @param[in]  
 *   @n mtu             Maximum Frame length to configure on the switch.
 *
 *   @return
 *   @n None
 * =============================================================================
 */
 unsigned char pause_frame_address[]={0x05,0x04,0x03,0x02,0x01,0x00};
void Init_Switch (Uint32 mtu)
{
    CSL_CPSW_3GF_PORTSTAT               portStatCfg;
    CSL_CPSW_3GF_STATS			cpswStat;
    CSL_CPSW_3GF_TSCNTL        timeSyncCntlCfg;
    /* Enable the CPPI port, i.e., port 0 that does all 
     * the data streaming in/out of EMAC.
     */
    CSL_CPSW_3GF_enablePort0 ();
    CSL_CPSW_3GF_disableVlanAware ();
	CSL_CPSW_3GF_setPort0RxMaxLen (mtu);
	#ifdef _CPSW_TEST
    CSL_CPSW_3GF_setPort0VlanReg (1, 0, 7);//set port 0(HOST port) default vlan id 1 and priority 7(higest)
    CSL_CPSW_3GF_setPortVlanReg(0,2,0,6);//set SGMII port 0 default vlan id 2 and priority 6
	CSL_CPSW_3GF_setPortVlanReg(1,3,0,5);//set SGMII port 1 default vlan id 3 and priority 5

	#ifdef _CPSW_WORK_MODE_1_
    hCpsw3gfRegs->P0_RX_PRI_MAP_REG = 0x77777777;//map all the packets which received by port 0 as header priority 7
	hCpgmacSlRegs->SL_PORT[0].PRI_MAP_REG = 0x66666666;//map all the packets which received by SGMII port 0 as header priority 6
	hCpgmacSlRegs->SL_PORT[1].PRI_MAP_REG = 0x55555555;//map all the packets which received by SGMII port 1 as header priority 5

	hCpsw3gfRegs->PORT_INFO_GROUP[0].P_TX_PRI_MAP_REG = 0x32222222;//SGMII port 0: map the packets which have header priority 7 to switch priority queue 3, others to switch priority queue 2
	hCpsw3gfRegs->PORT_INFO_GROUP[1].P_TX_PRI_MAP_REG = 0x32222222;//SGMII port 1: map the packets which have header priority 7 to switch priority queue 3, others to switch priority queue 2

	#endif

	#ifdef _CPSW_WORK_MODE_2_

    hCpsw3gfRegs->P0_RX_PRI_MAP_REG = 0x77776666;//map all the packets which received by port 0 as header priority 7 and 6
	hCpgmacSlRegs->SL_PORT[0].PRI_MAP_REG = 0x55554444;//map all the packets which received by SGMII port 0 as header priority 5 and 4
	hCpgmacSlRegs->SL_PORT[1].PRI_MAP_REG = 0x33332222;//map all the packets which received by SGMII port 1 as header priority 3 and 2

	hCpsw3gfRegs->PORT_INFO_GROUP[0].P_TX_PRI_MAP_REG = 0x32100000;//SGMII port 0: map the packets which have header priority 7 to switch priority queue 3,header priority 6 to switch priority queue 2, 5->1;4,3,2,1->0
	hCpsw3gfRegs->PORT_INFO_GROUP[1].P_TX_PRI_MAP_REG = 0x32100000;//SGMII port 0: map the packets which have header priority 7 to switch priority queue 3,header priority 6 to switch priority queue 2, 5->1;4,3,2,1->0

	#endif

    hCpsw3gfRegs->P0_RX_PRI_MAP_REG = 0x77766655;//map all the packets which received by port 0 as header priority 7 6 and 5
	hCpgmacSlRegs->SL_PORT[0].PRI_MAP_REG = 0x44433322;//map all the packets which received by SGMII port 0 as header priority 4,3 and 2
	hCpgmacSlRegs->SL_PORT[1].PRI_MAP_REG = 0x11110000;//map all the packets which received by SGMII port 1 as header priority 1 and 0

	hCpsw3gfRegs->PORT_INFO_GROUP[0].P_TX_PRI_MAP_REG = 0x32100000;//SGMII port 0: map the packets which have header priority 7 to switch priority queue 3,header priority 6 to switch priority queue 2, 5->1;4,3,2,1->0
	hCpsw3gfRegs->PORT_INFO_GROUP[1].P_TX_PRI_MAP_REG = 0x32100000;//SGMII port 0: map the packets which have header priority 7 to switch priority queue 3,header priority 6 to switch priority queue 2, 5->1;4,3,2,1->0

	#endif
    /* Enable statistics on both the port groups:
     *
     * MAC Sliver ports -   Port 1, Port 2
     * CPPI Port        -   Port 0
     */
    portStatCfg.p0AStatEnable   =   1;
    portStatCfg.p0BStatEnable   =   1;
    portStatCfg.p1StatEnable    =   1;
    portStatCfg.p2StatEnable    =   1;
    CSL_CPSW_3GF_setPortStatsEnableReg (&portStatCfg);

    /* Setup the Address Lookup Engine (ALE) Configuration:
     *      (1) Enable ALE.
     *      (2) Clear stale ALE entries.
     *      (3) Disable VLAN Aware lookups in ALE since
     *          we are not using VLANs by default.
     *      (4) No Flow control
     *      (5) Configure the Unknown VLAN processing 
     *          properties for the switch, i.e., which 
     *          ports to send the packets to.
     */
    CSL_CPSW_3GF_enableAle ();
    CSL_CPSW_3GF_clearAleTable ();

    //CSL_CPSW_3GF_enableVlanAware();
    //CSL_CPSW_3GF_enableAleVlanAware();
    //CSL_CPSW_3GF_disableAleVlanAware ();
    CSL_CPSW_3GF_disableAleTxRateLimit ();
    CSL_CPSW_3GF_setAlePrescaleReg (125000000u/1000u);
    CSL_CPSW_3GF_setAleUnkownVlanReg (7, 3, 3, 0);
    //CSL_CPSW_3GF_setAleUnkownVlanReg (0, 0, 0, 0);
    //if(cpswLpbkMode != CPSW_LOOPBACK_NONE)
    // CSL_CPSW_3GF_enableAleBypass();

     CSL_CPSW_3GF_disableAleBypass();

	//CSL_CPSW_3GF_setPortMaxBlocksReg(0,8,12);//SGMII0 block assignment
	//CSL_CPSW_3GF_setPortMaxBlocksReg(1,8,12);//SGMII1 block assignment
	CSL_CPSW_3GF_setPortMACAddress(1,pause_frame_address);
	//CSL_CPSW_3GF_setPortMaxBlocksReg(1,8,12);
	//CSL_CPSW_3GF_enableFIFOLoopback();

	CSL_CPSW_3GF_getStats(&cpswStat);

	//below is the configuration for IEEE1588 annex F packet detection/classification
	/*IEEE1588 packet detect enabled*/
	memset(&timeSyncCntlCfg,0,sizeof(CSL_CPSW_3GF_TSCNTL));
	timeSyncCntlCfg.tsRxEnable = 1;
	//timeSyncCntlCfg.tsTxVlanLType1Enable = 1;
	//timeSyncCntlCfg.tsTxVlanLType2Enable = 1;
	//timeSyncCntlCfg.tsRxVlanLType1Enable = 1;
	//timeSyncCntlCfg.tsRxVlanLType2Enable = 1;
	timeSyncCntlCfg.tsTxEnable = 1;
	timeSyncCntlCfg.txMsgTypeEnable = 0xffff;
	
	CSL_CPSW_3GF_setPortTimeSyncCntlReg (0,&timeSyncCntlCfg);
	CSL_CPSW_3GF_setPortTimeSyncCntlReg (1,&timeSyncCntlCfg);

	CSL_CPSW_3GF_setPortTimeSyncSeqIdReg(0,0x88f7,0x1e);
	CSL_CPSW_3GF_setPortTimeSyncSeqIdReg(1,0x88f7,0x1e);

	CSL_CPTS_disableCpts();
	CSL_CPTS_setRFTCLKSelectReg(1);
	CSL_CPTS_enableCpts();
	CSL_CPTS_disableInterrupt();
	Cpsw_ALE_config();
    /* Done with switch configuration */
    return;
}


/** ============================================================================
 *   @n@b Switch_update_addr
 *
 *   @b Description
 *   @n This API add/delete entries in the Address Lookup Engine (ALE) in "Switch" mode.
 *
 *   @param[in]  
 *   @n portNum         Switch port number.
 
 *   @param[in]  
 *   @n macAddress      MAC address to configure on the switch.
 * 
 *   @param[in]  
 *   @n add             0:add; 1:delete.
 *
 *   @return
 *   @n None
 *
 *   @Note  It supports "add" operation only now.           
 * =============================================================================
 */
int Switch_update_addr (Uint32 portNum, Uint8 macAddress[6], Uint16 add)
{
    Uint32                              i;
    CSL_CPSW_3GF_ALE_PORTCONTROL        alePortControlCfg;
    CSL_CPSW_3GF_ALE_UNICASTADDR_ENTRY  ucastAddrCfg;


    /* Configure the address in "Learning"/"Forward" state */
    alePortControlCfg.portState             =   ALE_PORTSTATE_FORWARD;
    alePortControlCfg.dropUntaggedEnable    =   0;
    alePortControlCfg.vidIngressCheckEnable =   0;
    //alePortControlCfg.noLearnModeEnable     =   (cpswLpbkMode != CPSW_LOOPBACK_NONE)?1:0;
	alePortControlCfg.noLearnModeEnable     =   1;
    alePortControlCfg.mcastLimit            =   0;
    alePortControlCfg.bcastLimit            =   0;

    CSL_CPSW_3GF_setAlePortControlReg (portNum, &alePortControlCfg);

    if (cpswLpbkMode != CPSW_LOOPBACK_NONE)
    {
        /* Program the ALE with the MAC address.
        *
        * The ALE entries determine the switch port to which any
        * matching received packet must be forwarded to.
        */
        /* Get the next free ALE entry to program */
        for (i = 0; i < CSL_CPSW_3GF_NUMALE_ENTRIES; i++)
        {
            if (CSL_CPSW_3GF_getALEEntryType (i) == ALE_ENTRYTYPE_FREE)
            {
                /* Found a free entry */
                break;                    
            }
        }
        if (i == CSL_CPSW_3GF_NUMALE_ENTRIES)
        {
            /* No free ALE entry found. return error. */            
            return -1;            
        }
        else
        {
            /* Found a free ALE entry to program our MAC address */            
            memcpy (ucastAddrCfg.macAddress, macAddress, 6);    // Set the MAC address
            ucastAddrCfg.ucastType      =      ALE_UCASTTYPE_UCAST_NOAGE;   // Add a permanent unicast address entryALE_UCASTTYPE_UCAST_NOAGE.
            ucastAddrCfg.secureEnable   =      FALSE;   
            ucastAddrCfg.blockEnable    =      FALSE;   
            ucastAddrCfg.portNumber     =      portNum;   // Add the ALE entry for this port

            /* Setup the ALE entry for this port's MAC address */
            CSL_CPSW_3GF_setAleUnicastAddrEntry (i, &ucastAddrCfg);            
        }
    }

    /* Done with upading address */
    return 0;
}


/** ============================================================================
 *   @n@b Init_SGMII_SERDES
 *
 *   @b Description
 *   @n This API sets up the configuration for the SGMII SERDES. Assumes a 125 MHz
 *       reference clock.
 *
 *   @param[in]  
 *   @n None
 * 
 *   @return
 *   @n None
 * =============================================================================
 */
Int32 Init_SGMII_SERDES(Void)
{

	CSL_SGMII_STATUS sgmii_status;
		
	/* Unlock the chip configuration registers to allow SGMII SERDES registers to
	* be written */
	CSL_BootCfgUnlockKicker();
	
	/* Configure the SERDES */
	/* Multiply to be 8 with Quarter Rate in the Rx registers */
	#ifdef TCI6614_EVM
	CSL_BootCfgSetSGMIIConfigPLL (0x00000041);
	#else //ZTE EVM
	CSL_BootCfgSetSGMIIConfigPLL (0x00000081);
	#endif
	TSC_delay_ms(1);

	//31:25    Reserved    0000000
	//23:24    LOOPBACK    00
	//	 22    ENOC 	   1
	//21:18    EQ		   0001
	//17:15    CDR		   001 -- first order threshold of 17
	//14:12    LOS		   000 -- tie off
	//11:10    ALIGN	   01  -- Comma Aligned
	//09:07    TERM 	   100 -- tie off (100)
	//	 06    INVPAIR	   0
	//05:04    RATE 	   01  -- tie off (10)	//00 = Full Rate, 01 = Half Rate (*0.5), 10 = Quarter Rate (*0.25)
	//03:01    BUSWIDTH    000 -- tie off
	//	 00    ENRX 	   1
	// 0000 0000 0100 0100 0000 0010 0001 0001 = 0x0044_0211 -- My estimated value
	// 0000 0000 0100 0100 0000 0100 0001 0001 = 0x0044_0411 -- New DV value
	// 0000 0000 0000 1000 0000 1000 0100 0001 = 0x0008_0841 -- Original DV value
	
	//CSL_BootCfgSetSGMIIRxConfig (0, 0x00700621);
	//CSL_BootCfgSetSGMIIRxConfig (1, 0x00700621);
	#ifdef _SerDes_LOOPBACK
	CSL_BootCfgSetSGMIIRxConfig (0, 0x01f00621);
	CSL_BootCfgSetSGMIIRxConfig (1, 0x01f00621); //SerDes RX loopback
	#else
	CSL_BootCfgSetSGMIIRxConfig (0, 0x00700621);
	CSL_BootCfgSetSGMIIRxConfig (1, 0x00700621);
	#endif
	//31:22    Reserved    0
	//21:20    LOOPBACK    00
	//19:18    RDTCT	   00  -- tie off
	//	 17    ENIDL	   0   -- tie off
	//	 16    MYSNC	   1   -- tie off
	//15:12    DEMPHASIS   ???? - 0001 Lets give some de-emphasis
	//11:08    SWING	   ????
	//	 07    CM		   1   -- tie off
	//	 06    INVPAIR	   0
	//05:04    RATE 	   01  -- tie off
	//03:01    BUSWIDTH    000 -- tie off
	//	 00    ENTX 	   1
	// 0000 0000 0011 0001 ???? ???? 1001 0001 = 0x0031_1E91 -- My estimated value
	// 0000 0000 0000 0001 0000 1111 0001 0001 = 0x0001_0F11 -- New DV value
	// 0000 0000 0100 0000 0001 1110 0100 0001 = 0x0040_1e41 -- Original DV value
	#ifndef _SerDes_LOOPBACK
	CSL_BootCfgSetSGMIITxConfig (0, 0x000108A1);	
	CSL_BootCfgSetSGMIITxConfig (1, 0x000108A1);
	#else
	CSL_BootCfgSetSGMIITxConfig (0, 0x002108A1);	
	CSL_BootCfgSetSGMIITxConfig (1, 0x002108A1); //SerDes TX loopback
	#endif
	/* Poll the SGMII0 lock bit to confirm that the sgmii module has recognized
	that the SERDES PLL has locked */	
	do
	{
		CSL_SGMII_getStatus(0, &sgmii_status);
	} while (sgmii_status.bIsLocked != 1);
	
	/* Poll the SGMII1 lock bit to confirm that the sgmii module has recognized
	that the SERDES PLL has locked */	
	do
	{
		CSL_SGMII_getStatus(1, &sgmii_status);
	} while (sgmii_status.bIsLocked != 1);
	/* Re-lock the chip configuration registers to prevent unintentional writes */
	CSL_BootCfgLockKicker();
/* SGMII SERDES Configuration complete. Return. */
	return 0;
}




/** ============================================================================
 *   @n@b Init_SGMII_SERDES
 *
 *   @b Description
 *   @n This API sets up the configuration for the SGMII SERDES. Assumes a 125 MHz
 *       reference clock.
 *
 *   @param[in]  
 *   @n None
 * 
 *   @return
 *   @n None
 * =============================================================================
 */
Int32 Init_SGMII_SERDES_by_port(Int32 portNum)
{

	CSL_SGMII_STATUS sgmii_status;
		
	/* Unlock the chip configuration registers to allow SGMII SERDES registers to
	* be written */
	CSL_BootCfgUnlockKicker();
	
	/* Configure the SERDES */
	/* Multiply to be 8 with Quarter Rate in the Rx registers */
	#ifdef TCI6614_EVM
	CSL_BootCfgSetSGMIIConfigPLL (0x00000041);
	#else //ZTE EVM
	CSL_BootCfgSetSGMIIConfigPLL (0x00000081);
	#endif
	TSC_delay_ms(1);

	//31:25    Reserved    0000000
	//23:24    LOOPBACK    00
	//	 22    ENOC 	   1
	//21:18    EQ		   0001
	//17:15    CDR		   001 -- first order threshold of 17
	//14:12    LOS		   000 -- tie off
	//11:10    ALIGN	   01  -- Comma Aligned
	//09:07    TERM 	   100 -- tie off (100)
	//	 06    INVPAIR	   0
	//05:04    RATE 	   01  -- tie off (10)	//00 = Full Rate, 01 = Half Rate (*0.5), 10 = Quarter Rate (*0.25)
	//03:01    BUSWIDTH    000 -- tie off
	//	 00    ENRX 	   1
	// 0000 0000 0100 0100 0000 0010 0001 0001 = 0x0044_0211 -- My estimated value
	// 0000 0000 0100 0100 0000 0100 0001 0001 = 0x0044_0411 -- New DV value
	// 0000 0000 0000 1000 0000 1000 0100 0001 = 0x0008_0841 -- Original DV value
	
	//CSL_BootCfgSetSGMIIRxConfig (0, 0x00700621);
	//CSL_BootCfgSetSGMIIRxConfig (1, 0x00700621);

	CSL_BootCfgSetSGMIIRxConfig (portNum, 0x00700621);
	//31:22    Reserved    0
	//21:20    LOOPBACK    00
	//19:18    RDTCT	   00  -- tie off
	//	 17    ENIDL	   0   -- tie off
	//	 16    MYSNC	   1   -- tie off
	//15:12    DEMPHASIS   ???? - 0001 Lets give some de-emphasis
	//11:08    SWING	   ????
	//	 07    CM		   1   -- tie off
	//	 06    INVPAIR	   0
	//05:04    RATE 	   01  -- tie off
	//03:01    BUSWIDTH    000 -- tie off
	//	 00    ENTX 	   1
	// 0000 0000 0011 0001 ???? ???? 1001 0001 = 0x0031_1E91 -- My estimated value
	// 0000 0000 0000 0001 0000 1111 0001 0001 = 0x0001_0F11 -- New DV value
	// 0000 0000 0100 0000 0001 1110 0100 0001 = 0x0040_1e41 -- Original DV value
	CSL_BootCfgSetSGMIITxConfig (portNum, 0x000108A1);	

	/* Poll the SGMII0 lock bit to confirm that the sgmii module has recognized
	that the SERDES PLL has locked */	
	do
	{
		CSL_SGMII_getStatus(portNum, &sgmii_status);
	} while (sgmii_status.bIsLocked != 1);
	
	CSL_BootCfgLockKicker();
/* SGMII SERDES Configuration complete. Return. */
	return 0;
}


Int32 SGMII_SERDES_RX_loopback(Void)
{
	CSL_BootCfgUnlockKicker();

	CSL_BootCfgSetSGMIIRxConfig (0, 0x01f00621);
	CSL_BootCfgSetSGMIIRxConfig (1, 0x01f00621);
	CSL_BootCfgLockKicker();

	return 0;
}

/** ============================================================================
 *   @n@b Init_Cpsw
 *
 *   @b Description
 *   @n This API sets up the entire ethernet subsystem and all its associated
 *      components.
 *
 *   @param[in]  
 *   @n None
 * 
 *   @return
 *   @n None
 * =============================================================================
 */
Int32 Init_Cpsw (Void)
{                      
    Uint32       macPortNum, mtu = 1520;
    Uint8        macSrcAddress [2][6] =  {{0x00, 0x27, 0x13, 0x69, 0x64, 0xa2},
                                          {0x00, 0x27, 0x13, 0x69, 0x64, 0xa1}};
    Uint8        macAddress0 [6] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05};  /* MAC address for (CPPI) Port 0 */ 
    Uint8        macAddress1 [6] = {0x10, 0x11, 0x12, 0x13, 0x14, 0x15};  /* MAC address for (EMAC1) Port 1 */ 
    Uint8        macAddress2 [6] = {0x20, 0x21, 0x22, 0x23, 0x24, 0x25};  /* MAC address for (EMAC2) Port 2 */
    Uint8        macAddress3 [6] = {0x01, 0x1b, 0x19, 0x00, 0x00, 0x00};  /* IEEE1588 MAC address for (CPPI) Port 0*/

		//powerDone_powerUp_EMAC();
	CSL_SGMII_disableAutoNegotiation (0);
	if(CSL_SGMII_getRxTxSoftResetStatus(0)==1)
		CSL_SGMII_endRxTxSoftReset(0);
	Init_SGMII_SERDES_by_port(0);
	Init_SGMII_u_boot(0);

	CSL_SGMII_disableAutoNegotiation (1);
	if(CSL_SGMII_getRxTxSoftResetStatus(1)==1)
		CSL_SGMII_endRxTxSoftReset(1); //make port 1 always reset
	Init_SGMII_SERDES_by_port(1);
	Init_SGMII_u_boot(1);

	    /* Setup the Ethernet switch finally. */
    	Init_Switch (mtu);

    
	Cpsw_configure_ALE_port(0);
	Cpsw_configure_ALE_port(1);
	Cpsw_configure_ALE_port(2);

	Init_MDIO();

    /* Initialize the SGMII/Sliver submodules for the 
     * two corresponding MAC ports.
     */
     tci6614_eth_gigabit_enable();

    for (macPortNum = 0; macPortNum < NUM_MAC_PORTS; macPortNum++)
    {
        Init_MAC_u_boot (macPortNum, &macSrcAddress[macPortNum][0], mtu);
    }



    return 0;    
}

void open_emac()
{
	Init_Cpsw();
	printf("open_emac!!\n");
}

void close_emac()
{
	int i;
	#if 0
	for(i=0;i<2;i++)
	{
		/* Reset MAC Sliver 0 */            
	    CSL_CPGMAC_SL_resetMac (i);
		//NO delay should cause issue
	    while (CSL_CPGMAC_SL_isMACResetDone (i) != TRUE);
	}
	#endif
	CSL_SGMII_startRxTxSoftReset (0);
	CSL_SGMII_startRxTxSoftReset (1);
	
	printf("close_emac!!\n");
}

void Cpsw_ALE_reset(void)
{
	printf("\n!!!!Begin to take the CPSW ALE reset!!!(64 cycles need)\n");
	CSL_CPSW_3GF_clearAleTable();
}

void Cpsw_ALE_AgeOut_UnTouch(void)
{
	printf("\n!!!!Begin to take the CPSW ALE Age Out UnTouch Process!!!\n");
	walk_through_ALE_for_reset_touch_flag();
	printf("\n!!!!CPSW ALE Age Out UnTouch Process Done!!!\n");
}


void Cpsw_ALE_AgeOut(void)
{
	printf("\n!!!!Begin to take the CPSW ALE Age Out Process!!!\n");
	CSL_CPSW_3GF_startAleAgeOutNow();
	while(!CSL_CPSW_3GF_isAleAgeOutDone())
	{
		printf(".");
	}
	printf("\n!!!!CPSW ALE Age Out Process Done!!!\n");
}

int Cpsw_configure_ALE_port (Uint32 portNum)
{
		CSL_CPSW_3GF_ALE_PORTCONTROL		alePortControlCfg;
	
	
		/* Configure the address in "Learning"/"Forward" state */
		alePortControlCfg.portState 			=	ALE_PORTSTATE_FORWARD;
		alePortControlCfg.dropUntaggedEnable	=	0;
		alePortControlCfg.vidIngressCheckEnable =	0;
		alePortControlCfg.noLearnModeEnable	  =   (cpswLpbkMode != CPSW_LOOPBACK_NONE)?1:0;
		alePortControlCfg.mcastLimit			=	0;
		alePortControlCfg.bcastLimit			=	0;
	
		CSL_CPSW_3GF_setAlePortControlReg (portNum, &alePortControlCfg);
		/* Done with upading address */
		return 0;
}

int Cpsw_disable_ALE_port (Uint32 portNum)
{
		CSL_CPSW_3GF_ALE_PORTCONTROL		alePortControlCfg;
	
	
		/* Configure the address in "Learning"/"Forward" state */
		alePortControlCfg.portState 			=	ALE_PORTSTATE_DISABLED;
		alePortControlCfg.dropUntaggedEnable	=	0;
		alePortControlCfg.vidIngressCheckEnable =	0;
		alePortControlCfg.noLearnModeEnable	  =   1;
		alePortControlCfg.mcastLimit			=	0;
		alePortControlCfg.bcastLimit			=	0;
	
		CSL_CPSW_3GF_setAlePortControlReg (portNum, &alePortControlCfg);

		return 0;
}


int Cpsw_ALE_add_addr (Uint32 portNum, Uint8 macAddress[6])
{
    Uint32                              i;
    CSL_CPSW_3GF_ALE_UNICASTADDR_ENTRY  ucastAddrCfg;

        /* Program the ALE with the MAC address.
        *
        * The ALE entries determine the switch port to which any
        * matching received packet must be forwarded to.
        */
        /* Get the next free ALE entry to program */
        for (i = 0; i < CSL_CPSW_3GF_NUMALE_ENTRIES; i++)
        {
            if (CSL_CPSW_3GF_getALEEntryType (i) == ALE_ENTRYTYPE_FREE)
            {
                /* Found a free entry */
                break;                    
            }
        }
        if (i == CSL_CPSW_3GF_NUMALE_ENTRIES)
        {
            /* No free ALE entry found. return error. */            
            return -1;            
        }
        else
        {
            /* Found a free ALE entry to program our MAC address */            
            memcpy (ucastAddrCfg.macAddress, macAddress, 6);    // Set the MAC address
            ucastAddrCfg.ucastType      =      ALE_UCASTTYPE_UCAST_NOAGE;   // Add a permanent unicast address entryALE_UCASTTYPE_UCAST_NOAGE.
            ucastAddrCfg.secureEnable   =      FALSE;   
            ucastAddrCfg.blockEnable    =      FALSE;   
            ucastAddrCfg.portNumber     =      portNum;   // Add the ALE entry for this port

            /* Setup the ALE entry for this port's MAC address */
            CSL_CPSW_3GF_setAleUnicastAddrEntry (i, &ucastAddrCfg);            
        }

    /* Done with upading address */
    return 0;
}


int Cpsw_ALE_add_multicastAddr (Uint32 portNum, Uint8 macAddress[6])
{
    Uint32                              i;
    CSL_CPSW_3GF_ALE_MCASTADDR_ENTRY  mcastAddrCfg;

        /* Program the ALE with the MAC address.
        *
        * The ALE entries determine the switch port to which any
        * matching received packet must be forwarded to.
        */
        /* Get the next free ALE entry to program */
        for (i = 0; i < CSL_CPSW_3GF_NUMALE_ENTRIES; i++)
        {
            if (CSL_CPSW_3GF_getALEEntryType (i) == ALE_ENTRYTYPE_FREE)
            {
                /* Found a free entry */
                break;                    
            }
        }
        if (i == CSL_CPSW_3GF_NUMALE_ENTRIES)
        {
            /* No free ALE entry found. return error. */            
            return -1;            
        }
        else
        {
            /* Found a free ALE entry to program our MAC address */            
            memcpy (mcastAddrCfg.macAddress, macAddress, 6);    // Set the MAC address
            mcastAddrCfg.mcastFwdState      =      0x11;   // Add a permanent unicast address entryALE_UCASTTYPE_UCAST_NOAGE.
            mcastAddrCfg.superEnable   =      FALSE;   
            mcastAddrCfg.portMask    =      1<<portNum;   

            /* Setup the ALE entry for this port's MAC address */
            CSL_CPSW_3GF_setAleMcastAddrEntry (i, &mcastAddrCfg);            
        }

    /* Done with upading address */
    return 0;
}


int Cpsw_ALE_add_vlan_addr (Uint32 portNum, Uint32 vlanid,Uint8 macAddress[6])
{
    Uint32                              i;
    CSL_CPSW_3GF_ALE_VLANUNICASTADDR_ENTRY  ucastAddrCfg;

        /* Program the ALE with the MAC address.
        *
        * The ALE entries determine the switch port to which any
        * matching received packet must be forwarded to.
        */
        /* Get the next free ALE entry to program */
        for (i = 0; i < CSL_CPSW_3GF_NUMALE_ENTRIES; i++)
        {
            if (CSL_CPSW_3GF_getALEEntryType (i) == ALE_ENTRYTYPE_FREE)
            {
                /* Found a free entry */
                break;                    
            }
        }
        if (i == CSL_CPSW_3GF_NUMALE_ENTRIES)
        {
            /* No free ALE entry found. return error. */            
            return -1;            
        }
        else
        {
            /* Found a free ALE entry to program our MAC address */            
            memcpy (ucastAddrCfg.macAddress, macAddress, 6);    // Set the MAC address
            ucastAddrCfg.ucastType      =      ALE_UCASTTYPE_UCAST_NOAGE;   // Add a permanent unicast address entryALE_UCASTTYPE_UCAST_NOAGE.
            ucastAddrCfg.vlanId = vlanid;
            ucastAddrCfg.secureEnable   =      FALSE;   
            ucastAddrCfg.blockEnable    =      FALSE;   
            ucastAddrCfg.portNumber     =      portNum;   // Add the ALE entry for this port

            /* Setup the ALE entry for this port's MAC address */
            CSL_CPSW_3GF_setAleVlanUnicastAddrEntry (i, &ucastAddrCfg);            
        }

    /* Done with upading address */
    return 0;
}

int Cpsw_ALE_add_vlan_config(Uint32 vlanid,Uint32 vlanMemList,Uint32 unRegMcastFloodMask,Uint32 regMcastFloodMask,Uint32 forceUntaggedEgress)
{
    Uint32                              i;
    CSL_CPSW_3GF_ALE_VLAN_ENTRY  vlanCfg;

        /* Program the ALE with the MAC address.
        *
        * The ALE entries determine the switch port to which any
        * matching received packet must be forwarded to.
        */
        /* Get the next free ALE entry to program */
        for (i = 0; i < CSL_CPSW_3GF_NUMALE_ENTRIES; i++)
        {
            if (CSL_CPSW_3GF_getALEEntryType (i) == ALE_ENTRYTYPE_FREE)
            {
                /* Found a free entry */
                break;                    
            }
        }
        if (i == CSL_CPSW_3GF_NUMALE_ENTRIES)
        {
            /* No free ALE entry found. return error. */            
            return -1;            
        }
        else
        {
            vlanCfg.vlanMemList = vlanMemList;
            vlanCfg.vlanId = vlanid;
            vlanCfg.unRegMcastFloodMask =      unRegMcastFloodMask;   
            vlanCfg.regMcastFloodMask    =      regMcastFloodMask;   
            vlanCfg.forceUntaggedEgress     =      forceUntaggedEgress; 

            /* Setup the ALE entry for this port's VLAN config */
            CSL_CPSW_3GF_setAleVlanEntry (i, &vlanCfg);            
        }

    /* Done with upading address */
    return 0;
}

